-- mega_ram2.vhd 
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use IEEE.std_logic_arith.all;

-- 	入出力の宣言
entity mega_ram2 is
	port (
		CLK 			: in std_logic;
		RAM_ADDR		: in std_logic_vector (7 downto 0);
		LRD_STR		: in std_logic;
		RAM_IN		: in std_logic_vector (15 downto 0);
		IO65_IN		: in std_logic_vector (15 downto 0);
		RAM_WEN		: in std_logic;
		RAM_OUT		: out std_logic_vector (15 downto 0);
		IO64_OUT		: out std_logic_vector (15 downto 0)
		);
end mega_ram2;

-- 回路の記述
architecture RTL of mega_ram2 is
-- コンポーネント ram_1port の宣言
component ram_1port
	port (
		address	: IN STD_LOGIC_VECTOR (5 DOWNTO 0);
		clock		: IN STD_LOGIC;
		data		: IN STD_LOGIC_VECTOR (15 DOWNTO 0);
		wren		: IN STD_LOGIC ;
		q			: OUT STD_LOGIC_VECTOR (15 DOWNTO 0)
	);
end component;
-- 内部信号の定義
signal ADDR_INT			: integer range 0 to 255;
signal ADDR_INT_DLY		: integer range 0 to 255;
signal RAM_1PORT_ADDR	: std_logic_vector(7 downto 0);
signal RAM_OUT_MEGA		: std_logic_vector(15 downto 0);
signal RAM_ADDR_DLY		: std_logic_vector(7 downto 0);
signal RAM_ADDR_DLY2		: std_logic_vector(7 downto 0);

begin
	ADDR_INT <= conv_integer(RAM_ADDR);
	ADDR_INT_DLY <= conv_integer(RAM_ADDR_DLY2);
-- コンポーネント ram_1port の実体化と入出力の相互接続
	C1: ram_1port port map(
			address => RAM_1PORT_ADDR(5 downto 0),
			clock => CLK,
			data => RAM_IN,
			wren => RAM_WEN,
			q => RAM_OUT_MEGA
		);
-- コンポーネント ram_1port の入力信号の生成(1)
	process(LRD_STR, RAM_ADDR, RAM_ADDR_DLY2)
	begin
		if(LRD_STR = '1') then
			RAM_1PORT_ADDR <= RAM_ADDR;
		else
			RAM_1PORT_ADDR <= RAM_ADDR_DLY2;
		end if;
	end process;
-- コンポーネント ram_1port の入力信号の生成(2)
	process(CLK)
	begin
		if(CLK'event and CLK = '1') then
			if(ADDR_INT < 64) then
				RAM_OUT <= RAM_OUT_MEGA;
			elsif(ADDR_INT = 65) then
				RAM_OUT <= IO65_IN;
			else
				RAM_OUT <= (others => '0');
			end if;				
-- I/O出力信号 IO64_OUT の生成
			if(ADDR_INT_DLY = 64) then
				if(RAM_WEN = '1') then
					IO64_OUT <= RAM_IN;
				end if;
			end if;
			RAM_ADDR_DLY <= RAM_ADDR;
			RAM_ADDR_DLY2 <= RAM_ADDR_DLY;
		end if;
	end process;
end RTL;
